import Vue from 'vue'

const regSeries = ['pie', 'sector', 'line', 'smoothline', 'dashline', 'area',
  'stackarea', 'smootharea', 'bar', 'stackbar', 'dodgebar', 'point', 'waterfall',
  'funnel', 'pyramid', 'radialbar', 'schema', 'box', 'candle', 'polygon', 'contour',
  'heatmap', 'edge'];

const camelCase = (function () {
  const DEFAULT_REGEX = /[-_]+(.)?/g;

  function toUpper(match, group1) {
    return group1 ? group1.toUpperCase() : '';
  }
  return function (str, delimiters) {
    return str.replace(delimiters ? new RegExp('[' + delimiters + ']+(.)?', 'g') : DEFAULT_REGEX, toUpper);
  };
})();

const baseChartComponent = {
  data() {
    if (this.$options._componentTag === 'v-chart') { // root: chart
      return {
        isViser: true,
        jsonForD2: {

        }
      };
    }

    if (this.$options._componentTag === 'v-views') { // root: views
      return {
        isViser: true,
        jsonForD2: {

        }
      };
    }

    // tool chart
    return {
      isViser: true,
    };
  },
  props: {
    width: Number,
    height: Number,
    data: Array | Object,
    dataDef: Array,
    dataPre: Object,
    crosshairs: Object,
    dataKey: String,
    label: Object | Boolean,
    size: Number,
    vStyle: Object,
    show: String | Boolean,
    color: Array | String,
    tooltip: String,
    opacity: Number,
    dataView: String,
    gemo: String,
    type: String
  },
  methods: {
    /**
     * find nearest parent rechart component
     */
    findNearestRootComponent(componentInstance) {
      if (componentInstance.isViser && ['v-chart', 'v-views'].indexOf(componentInstance.$options._componentTag) > -1) {
        return componentInstance
      }
      if (componentInstance.$parent) {
        return this.findNearestRootComponent(componentInstance.$parent)
      }
      return null
    }
  },
  created: function () { // bubble from parent to child

  },
  mounted: function () { // bubble from child to parent
    if (this.$options._componentTag === 'v-chart') { // hit top
      const d2Json = {
        data: this.data,
        dataDef: this.dataDef,
        dataPre: this.dataPre,
        dataView: this.dataView,
        ...this.jsonForD2
      }

      const code = document.createElement('div')
      code.style.whiteSpace = 'pre'
      code.style.textAlign = 'left'
      code.style.fontSize = '12px'
      code.style.lineHeight = '12px'
      code.textContent = JSON.stringify(d2Json, null, 4)
      this.$el.appendChild(code)
    } else if (this.$options._componentTag === 'v-views') {
      const nearestRootComponent = this.findNearestRootComponent(this.$parent)

      nearestRootComponent.jsonForD2.views = {
        data: this.data,
        dataDef: this.dataDef,
        dataPre: this.dataPre,
        dataView: this.dataView,
        ...this.jsonForD2
      }
    } else {
      const nearestRootComponent = this.findNearestRootComponent(this.$parent)

      if (!nearestRootComponent) {
        throw Error(`${this.$options._componentTag} must be wrapped into v-chart`)
      }

      const rechartName = this.$options._componentTag.replace(/-/g, '').slice(1)
      const rechartNameCamelCase = camelCase(this.$options._componentTag.slice(2))

      if (isAllUndefined(this._props)) {
        nearestRootComponent.jsonForD2[rechartName] = true
      } else if (regSeries.indexOf(rechartName) > -1) {
        safePush(nearestRootComponent.jsonForD2, 'series', {
          quickType: rechartNameCamelCase,
          ...cleanUndefined(normalizeProps(this._props))
        })
      } else {
        oneObjectMoreArray(nearestRootComponent.jsonForD2, rechartName, cleanUndefined(normalizeProps(this._props)))
      }
    }
  },
  updated: function () {

  },
  render(h) {
    return h('div', null, this.$slots.default)
  }
}

export default {
  install: (Vue, options) => {
    Vue.component('v-chart', baseChartComponent)
    Vue.component('v-smooth-line', baseChartComponent)
    Vue.component('v-point', baseChartComponent)
    Vue.component('v-tooltip', baseChartComponent)
    Vue.component('v-legend', baseChartComponent)
    Vue.component('v-axis', baseChartComponent)
    Vue.component('v-views', baseChartComponent)
    Vue.component('v-bar', baseChartComponent)
    Vue.component('v-schema', baseChartComponent)
    Vue.component('v-line', baseChartComponent)
    Vue.component('v-coord', baseChartComponent)
    Vue.component('v-pie', baseChartComponent)
    Vue.component('v-edge', baseChartComponent)
    Vue.component('v-series', baseChartComponent)
  }
}

function safePush(obj, key, value) {
  if (!obj[key]) {
    obj[key] = []
  }

  cleanUndefined(value)

  obj[key].push(value)
}

function oneObjectMoreArray(obj, key, value) {
  if (!obj[key]) {
    obj[key] = value
    return
  }

  if (obj[key] && obj[key].constructor.name === 'Object') {
    obj[key] = [obj[key]]
  }

  obj[key].push(value)
}

function cleanUndefined(value) {
  // delete value's undefined key
  for (const key in value) {
    if (value[key] === undefined) {
      delete value[key]
    }
  }

  return value
}

function isAllUndefined(value) {
  return Object.keys(value).every(key => value[key] === undefined)
}

function camelize(str) {
  return str.replace(/(?:^\w|[A-Z]|\b\w)/g, function (letter, index) {
    return index == 0 ? letter.toLowerCase() : letter.toUpperCase();
  }).replace(/\s+/g, '');
}

/**
 * special props for vue
 */
function normalizeProps(props) {
  const newProps = { ...props }

  if (newProps.vStyle) {
    newProps.style = newProps.vStyle
    delete newProps.vStyle
  }

  return newProps
}
